/*	Application		IPC sender-side using socket (UDP)		
	File			IPC_socket_Sender.java
	Purpose			Demonstration of local IPC, using a blocking socket
	Author			Richard John Anthony
	Date			January 2015
	
	Function		1. Creates a socket
				2. Waits for the user to type a message
                                3. sends the message to port (8007) on the local computer
				4. Exits once a key is pressed

    To run the project from the command line, go to the dist folder and type: java -jar "IPC_socket_Sender.jar"
*/
package ipc_socket_sender;

import java.io.Console;
import java.net.*;

public class IPC_socket_Sender 
{
    private static  int SEND_BUFFER_SIZE = 100;
    private static  int SEND_PORT = 8007; // The port to send to
    // Use a different port for binding to, so both sender and receiver processes can run on the same computer
    private static  int LOCAL_PORT = 8000; 
    private static DatagramSocket m_IPC_SendSOCKET;
    static Console con;
    
    public static void main(String[] args)
    {
        con = System.console();
        if (null == con)
        {
            System.out.println("!! Error accessing console: Run in a command window !!");
            System.exit(-1);
        }

        System.out.println("Local Inter-Process Communication (sender side).");
        System.out.println("This program waits for the user to type a message,");
        System.out.println("and sends it to port 8007 on the local computer.");
        System.out.println("--------------------------------------------------");
        System.out.println("Type a  message to send ...   ");

        IPC_socket_Sender Socket_Sender = new IPC_socket_Sender();
    }

    IPC_socket_Sender()
    {
        // Create a socket - this is the logical 'communication endpoint'
        CreateSendSocket();

        // Wait for the user to type a message
        String sMessageString = con.readLine(); // Get a string which may include spaces();
        
        // Send the message that the user typed, to the sepcified port on the local computer
        SendMessage(sMessageString);
        System.out.printf("   Message \"%s\" sent\n", sMessageString);
        CloseSocketAndExit();
    }

    void CreateSendSocket()
    {
        try
        {
            m_IPC_SendSOCKET = new DatagramSocket(LOCAL_PORT); // Create socket and bind it a specified port (uses the local address)
            m_IPC_SendSOCKET.setSoTimeout(100); // Non-blocking with a timeout value of 100 milliseconds
        }
        catch (SocketException ex)  // Catch exceptions from the socket operations
        {
            System.out.printf("Error %s\n", ex.toString());
            CloseSocketAndExit();
        }
    }
    
    void SendMessage(String sSendString)
    {
        byte[] bSendBuf = new byte[SEND_BUFFER_SIZE];
        bSendBuf = sSendString.getBytes();
        try
        {
            InetAddress SendIPAddress = InetAddress.getLoopbackAddress(); // Assume receiver is at same address as sender
            DatagramPacket SendPacket = new DatagramPacket(bSendBuf, sSendString.length(),
                                    SendIPAddress /*The address to send to*/, SEND_PORT); 
            m_IPC_SendSOCKET.send(SendPacket);
        }
        catch(Exception ex)
        {
            System.out.printf("Send failed: %s\n", ex.toString());
            CloseSocketAndExit();
        }
    }

    void CloseSocketAndExit()
    {
        try
        {
            m_IPC_SendSOCKET.close();
            System.out.println("Socket closed");
        }
        catch(Exception ex)
        {
        }
        System.out.println("(press a key to continue)");
        con.readLine();	// Wait for user to type a key before the program exits
        System.exit(1);
    }
}