/*	Application		Socket-based IPC threads example	
	File			Send_Thread.java
	Purpose			Demonstration of local IPC, and multithreading using a blocking socket
	Author			Richard John Anthony
	Date			February 2015
*/

package Multithreaded_IPC_two_worker_threads;

import java.net.DatagramPacket;
import java.net.DatagramSocket;
import java.net.InetAddress;
import java.net.SocketException;

public class Send_Thread  implements Runnable
{
    Thread m_SendThread;
    
    // Dedicated flags are used, one to run/stop each thread, to prevent access races
    public static Boolean m_bSendThread_Running = false;
    
    Send_Thread()
    {
        m_SendThread = new Thread(this); // Create the sender thread
    }

    public void Start()
    {   // The means of externally calling the thread's start() method (which invokes run()).
        m_SendThread.start();
    }

    public void run()
    {
        String sIn = "";
        while (true == m_bSendThread_Running)
        {
            System.out.println("SendThread: Type a message to send (\"Quit\" to quit): ");
            try
            {
                sIn = Multithreaded_IPC_two_worker_threads.con.readLine();
                if(sIn.length() >= 4 && sIn.substring(0, 4).toUpperCase().equals("QUIT"))
                {
                    System.out.println("Quiting");
                    Multithreaded_IPC_two_worker_threads.CloseSocketsAndExit(false);
                }
            }
            catch (Exception Ex)
            {   // Control-C for example causes us to come here
                System.out.println("Exception occurred, Quiting");
                Multithreaded_IPC_two_worker_threads.CloseSocketsAndExit(false);
            }
            int iNumberOfBytesSent = SendMessage(Multithreaded_IPC_two_worker_threads.m_iSendPort, sIn);
            if(-1 != iNumberOfBytesSent)
            {
                System.out.printf("SendThread  Sent: %s Number of bytes sent: %d\n", sIn, iNumberOfBytesSent);
            }
        }
    }

    static int SendMessage(int iSendPort, String sSendString)
    {
        byte[] bSendBuf = new byte[Multithreaded_IPC_two_worker_threads.BUFFER_SIZE];
        bSendBuf = sSendString.getBytes();
        int iNumBytesSent = 0;
        try
        {
            InetAddress IPAddress = InetAddress.getLocalHost();
            // Uncomment the statement below for diagnostic purposes
            // System.out.printf("Send assumes local receiver, uses local address: %s:%d\n", 
            //                                      IPAddress.toString(), iSendPort);
            DatagramPacket SendPacket = new DatagramPacket(bSendBuf, sSendString.length(),
                    IPAddress /*uses local address as the address to send to*/, iSendPort); 
            Multithreaded_IPC_two_worker_threads.m_IPC_SendSOCKET.send(SendPacket);
            iNumBytesSent = SendPacket.getLength();  
        }
        catch(Exception Ex)
        {
            System.out.printf("Send failed: %s\n", Ex.toString());
            return -1; // Signal that an error occurred
        }
        return iNumBytesSent;
    }
}