/*	Application		Periodic Output
	File			PeriodicOutput.Java 
	Purpose			Demonstration of command-line configurable program, with periodic output
	Author			Richard John Anthony
	Date			January 2015
	
	Function	1. Reads command-line parameters which configure the program's behaviour
                        2. Outputs a user-specified character
                        3. Sleeps a user-defined period
                        4. Repeats in loop for as many iterations as necessary until the total run time reaches 25 seconds

        To run the project from the command line, go to the dist folder and type: java -jar "PeriodicOutput.jar" 
*/

package PeriodicOutput;

public class PeriodicOutput
{
    public static void main(String[] args)
    {
        int iInterval = 1;
        int iTotalRunTime_Milliseconds = 25000;		// Program will run for 25 seconds
        int iAccumulatedRunTime_Milliseconds = 0;	// Run time accumulated
        String sCharToDisplay = "";
        
        if (2 != args.length)
        {   // User did not provide the required parameters

            // Note that in C++ the executable name is provided as command line argument [0]. 
            // In Java only the user-provided arguments are passed to the process in the args array,
            // (i.e. NOT including the executable name), so a different technique is necessary to get the program name.
            String sProgramName = new java.io.File(PeriodicOutput.class.getProtectionDomain()
                                                .getCodeSource().getLocation().getPath()).getName();
            System.out.printf("Usage: %s milliseconds_period character_to_print\n", sProgramName);
            System.exit(1);
        }
        try 
        {
            iInterval = Integer.parseInt(args[0]);
            if (0 >= iInterval)
            {   // iInterval too small (less than 1)
                System.out.println("Interval must be greater or equal to 1 millisecond");
                System.exit(1);
            }

            String sArg = args[1];
            if(sArg.length() >= 1)
            {
                sCharToDisplay = sArg.substring(0,1); // Get the first of possibly many chars that constitute the second argument
            }
        }
        catch (NumberFormatException Ex)
        {
            System.out.println("Mal-formed arguments (expecting milliseconds_period character_to_print)");
            System.exit(1);
        }

        while (iAccumulatedRunTime_Milliseconds < iTotalRunTime_Milliseconds)
        {
            System.out.print(sCharToDisplay);
            try
            {
                Thread.sleep(iInterval);
            }
            catch (InterruptedException Ex)
            {
            }
            iAccumulatedRunTime_Milliseconds += iInterval;
        }
    }
}