/*	Application		Threads Demonstration - Threads executed synchronously		
	File			Threads_Detach.Java 
	Purpose			Demonstrates the basic concepts of multi-threading
	Author			Richard John Anthony
	Date			February 2015
	
	Function		1. Creates three threads
                                2. Runs the threads asynchronously - threads run by default in 'detached' mode
                                3. Threads display text messages for diagnostic purposes

    To run from the command line, go to the dist folder and type: java -jar "Threads_Detach.jar"
*/

public class Threads_Detach implements Runnable
{
    Thread m_thread;
    private String m_sDisplayMessage;

    Threads_Detach(String sDisplayMessage)
    {
        m_sDisplayMessage = sDisplayMessage;
        m_thread = new Thread(this); // Create the thread
    }

    public void Start()
    {   // The means of externally calling the thread's start() method (which invokes run()).
        m_thread.start();
    }

    public void run()
    {
        try 
        {
            for (int iCount = 0; iCount < 10; iCount++)
            {
                System.out.printf("Thread ID:%d  Name:%s\n", m_thread.getId(), m_sDisplayMessage);
                Thread.sleep(30);
            }
        }
        catch (InterruptedException Ex) 
        {
            System.out.printf("Thread:%d was interrupted:%s\n", m_thread.getId(), Ex.toString());
        }
        System.out.printf("Thread %d is finished\n", m_thread.getId());
    }

    public static void main(String args[])
    {
        Threads_Detach T1 = new Threads_Detach("AAAAAAAAAA");
        T1.Start();

        Threads_Detach T2 = new Threads_Detach("BBBBBBBBBB");
        T2.Start();

        Threads_Detach T3 = new Threads_Detach("CCCCCCCCCC");
        T3.Start();

        // The Main thread
        for (int iCount = 0; iCount < 10; iCount++)
        {
            System.out.println("Main");
            try
            {
                Thread.sleep(30);
            }
            catch (InterruptedException Ex) 
            {
                System.out.printf("Main() was interrupted:%s\n", Ex.toString());
            }
        }
    }
}