/*	Application		Threads Demonstration - Threads executed synchronously		
	File			Threads_Join.Java 
	Purpose			Demonstrates the basic concepts of multi-threading
	Author			Richard John Anthony
	Date			February 2015
	
	Function		1. Creates three threads
                                2. Runs the threads synchronously - using join() - the calling thread waits for the called thread to terminate
                                3. Threads display text messages for diagnostic purposes

    To run from the command line, go to the dist folder and type: java -jar "Threads_Join.jar"
*/

public class Threads_Join implements Runnable
{
    Thread m_thread;
    private String m_sDisplayMessage;

    Threads_Join(String sDisplayMessage)
    {
        m_sDisplayMessage = sDisplayMessage;
        m_thread = new Thread(this); // Create the thread
    }

    public void Start()
    {   // The means of externally calling the thread's start() method (which invokes run()).
        m_thread.start();
    }

    public void run()
    {
        try 
        {
            for (int iCount = 0; iCount < 10; iCount++)
            {
                System.out.printf("Thread ID:%d  Name:%s\n", m_thread.getId(), m_sDisplayMessage);
                Thread.sleep(30);
            }
        }
        catch (InterruptedException Ex) 
        {
            System.out.printf("Thread:%d was interrupted:%s\n", m_thread.getId(), Ex.toString());
        }
        System.out.printf("Thread %d is finished\n", m_thread.getId());
    }

    public static void main(String args[])
    {
        Threads_Join T1 = new Threads_Join("AAAAAAAAAA");
        T1.Start();
        try
        {
            T1.m_thread.join();
        }
        catch (InterruptedException Ex)
        {
        }      

        Threads_Join T2 = new Threads_Join("BBBBBBBBBB");
        T2.Start();
        try
        {
            T2.m_thread.join();
        }
        catch (InterruptedException Ex)
        {
        }      

        Threads_Join T3 = new Threads_Join("CCCCCCCCCC");
        T3.Start();
        try
        {
            T3.m_thread.join();
        }
        catch (InterruptedException Ex)
        {
        }      

        // The Main thread
        for (int iCount = 0; iCount < 10; iCount++)
        {
            System.out.println("Main");
            try
            {
                Thread.sleep(30);
            }
            catch (InterruptedException Ex) 
            {
                System.out.printf("Main() was interrupted:%s\n", Ex.toString());
            }
        }
    }
}